#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "vpass.h"
#include "vp_err.h"
#include "vp_net.h"

static char cvs_id[] __unused =
"$Id: vpass.c,v 1.2 2001/08/21 13:05:34 roam Exp $";

int		verbose, quiet;

char		vp_verstr[64] = "vpass";

char		*vp_user, *vp_cuser, *vp_oldp, *vp_newp;

static int	vp_showprompt = 1;
static int	vp_auto;

static int	vp_netinit;
static int	vp_fds[2];

static vp_err_t	vp_init(int, char *[]);
static vp_err_t	vp_close(void);
static vp_err_t	vp_doit(int, char *[]);

static vp_err_t	vp_readvar(char **pval, const char *preset, const char *prompt,
		int pass);

static void	usage(void);
static vp_err_t	version(void);

/*
 * Function:
 *         vp_makeversion()                - generate version string
 * Inputs:
 *         none
 * Returns:
 *         vp_err_t
 * Modifies:
 *         vp_verstr
 */

static vp_err_t
vp_makeversion(void) {

	snprintf(vp_verstr, sizeof(vp_verstr), "vpass v%d.%d"
#if VP_VER_PRE
	    "-pre%d"
#endif /* VP_VER_PRE */
#if VP_VER_PATCH
	    "p%d"
#endif /* VP_VER_PATCH */
	    ,
	    VP_VER_MAJ, VP_VER_MIN
#if VP_VER_PRE
	    , VP_VER_PRE
#endif /* VP_VER_PRE */
#if VP_VER_PATCH
	    , VP_VER_PATCH
#endif /* VP_VER_PATCH */
		);
	
	return (VP_ERR_NONE);
}

/*
 * Function:
 * 	vp_init				- general-purpose init function
 * Inputs:
 * 	argc, argv			- main() args for option processing
 * Returns:
 * 	vp_err_t
 * 	CMDLINE
 * Modifies:
 * 	verbose, quiet
 */

static vp_err_t
vp_init(int argc, char *argv[]) {
	int ch, helpq, versq;
	
	helpq = versq = 0;
	vp_makeversion();
	
	/* Process cmdline options */
	
	opterr = 0;
	while (ch = getopt(argc, argv, VP_OPTSTR), ch != EOF)
		switch (ch) {
			case 'a':
				vp_auto = 1;
				break;
			case 'c':
				vp_cuser = optarg;
				break;
			case 'h':
				helpq = 1;
				break;
			case 'N':
				vp_showprompt = 0;
				break;
			case 'n':
				vp_newp = optarg;
				break;
			case 'p':
				vp_oldp = optarg;
				break;
			case 'q':
				quiet++;
				break;
			case 'u':
				vp_user = optarg;
				break;
			case 'V':
				versq = 1;
				break;
			case 'v':
				verbose++;
				break;
			case '?':
			default:
				usage();
				/* NOTREACHED */
		}
	
	argc -= optind; argv += optind;
	
	if (versq)
		version();
	if (helpq)
		usage();
	if (versq)
		/* no need for "|| helpq": usage() never returns */
		exit(VP_ERR_NONE);
	
	if (vp_auto && !(vp_user && vp_oldp && vp_newp))
		usage();

	if (vp_net_mode == VP_NET_NULL)
		vp_net_mode = VP_NET_UCSPI;

	return (VP_ERR_NONE);
}

/*
 * Function:
 * 	vp_close			- general-purpose shutdown function
 * Inputs:
 * 	none
 * Returns:
 * 	vp_err_t
 * 	nothing for the present (FIXME)
 * Modifies:
 * 	nothing for the present (FIXME)
 */

static vp_err_t
vp_close(void) {
	vp_err_t r;
	
	if (vp_netinit)
		if (r = vp_net_close(vp_fds), r)
			return (r);
	return (VP_ERR_NONE);
}

/*
 * Function:
 * 	usage			- startup help info
 * Inputs:
 * 	none
 * Returns:
 * 	nothing
 * Modifies:
 * 	nothing, writes to stdout and exits
 */

void
usage(void) {
	static const char *msg[] = {
	    "usage: vpass [-hqVv]",
	    "\t-h\tprint this help text and exit;",
	    "\t-q\tquiet operation; multiple -q's make it even more quiet;",
	    "\t-V\tprint version information and exit;",
	    "\t-v\tverbose operation; multiple -v's increase verbosity level.",
	    NULL
	};
	unsigned i;
	
	for (i = 0; msg[i] != NULL; i++)
		fprintf(stderr, "%s\n", msg[i]);
	exit(VP_ERR_CMDLINE);
}

/*
 * function:
 * 	vp_version		- output version info
 * Inputs:
 * 	none
 * Returns:
 * 	VP_ERR_NONE
 * Modifies:
 * 	nothing, writes to stdout
 */

vp_err_t
version(void) {
	
	printf("%s\n", vp_verstr);
	
	if (verbose) {
		printf("%s\n", "Built on " __DATE__ ", " __TIME__);
#ifdef __GNUC__
		if (verbose > 1)
			printf("%s\n", "Compiler: GNU C " __VERSION__
			    " on " VP_OS " " VP_OSREL ": " VP_OSHOST);
#endif /* __GNUC__ */
	}

	return (VP_ERR_NONE);
}

/*
 * Function:
 * 	vp_doit				- perform the actual work
 * Inputs:
 * 	argc, argv			- main() args
 * Returns:
 * 	vp_err_t
 * 	nothing so far (FIXME)
 * Modifies:
 *	nothing by itself; writes args to stdout
 */

static vp_err_t
vp_doit(int argc __unused, char *argv[] __unused) {
	vp_err_t r;
	
	vp_readvar(&vp_user, vp_user, "Username: ", 0);
	vp_readvar(&vp_oldp, vp_oldp, "Password: ", 1);
	vp_readvar(&vp_newp, vp_newp, "New password: ", 1);

	RDBG(("Attempting to change %s's password from %s to %s\n",
	    vp_user, vp_oldp, vp_newp));

	if (r = vp_net_connect(vp_fds), r)
		return (r);
	vp_netinit = 1;

	if (r = vp_net_talk(vp_fds), r)
		return (r);

	return (VP_ERR_NONE);
}

static vp_err_t
vp_readvar(char **pval, const char *preset, const char *prompt, int pass) {
	char *val;
	char buf[250];
	size_t len;

	if (preset == NULL) {
		if (vp_auto)
			return (VP_ERR_INT);

		if (pass) {
			preset = getpass(prompt);
		} else {
			if (vp_showprompt && (prompt != NULL))
				printf("%s", prompt);
			/* FIXME: dynamic? */
			if (fgets(buf, sizeof(buf), stdin) == NULL)
				return (VP_ERR_FGETS);
			preset = buf;
		}
	}

	if (val = strdup(preset), val == NULL)
		return (VP_ERR_NOMEM);
	len = strlen(val);
	while ((len > 0) && ((val[len - 1] == '\n') || (val[len - 1] == '\r')))
		val[--len] = '\0';

	*pval = val;
	return (VP_ERR_NONE);
}

/*
 *   M A I N   F U N C T I O N
 */

int
main(int argc, char *argv[]) {
	vp_err_t r, cr;
	
	if (r = vp_init(argc, argv), r)
		return (vp_prerror("init", r));
	
	argc -= optind;
	argv += optind;
	if (r = vp_doit(argc, argv), r)
		vp_prerror("doit", r);
	
	if (cr = vp_close(), cr)
		vp_prerror("close", cr);
	
	return (r? r: cr);
}
